-- frmMainWidget.lua
-- Originally TheOneBagWindow.lua written by Rod, extended by Habna


import "HabnaPlugins.HugeBag.GlobalVariableWidget"; -- LUA GlobalVariable file
import "HabnaPlugins.HugeBag.settingsWidget"; -- LUA settings file
import "HabnaPlugins.HugeBag.functionsWidget"; -- LUA functions file
import "HabnaPlugins.HugeBag.color"; -- LUA Color file
import "HabnaPlugins.HugeBag.sortingList"; -- LUA SortingList file
import "HabnaPlugins.HugeBag.moveWidget"; -- LUA MoveWidget file
import "HabnaPlugins.HugeBag.optionsWidget"; -- LUA options file


local resources = {
	  Item = { Background = "HabnaPlugins/HugeBag/Resources/slotBackground.tga" },
	  TopLeftCorner = { Background = "HabnaPlugins/HugeBag/Resources/topLeftCorner.tga" },
	  MidLeftBar = { Background = "HabnaPlugins/HugeBag/Resources/midLeftBar.tga" },
	  BotLeftCorner = { Background = "HabnaPlugins/HugeBag/Resources/botLeftCorner.tga" },
	  HugeBagName = { Image = "HabnaPlugins/HugeBag/Resources/label.tga" },
	  TopRightCorner = { Background = "HabnaPlugins/HugeBag/Resources/topRightCorner.tga" },
	  MidRightBar = { Background = "HabnaPlugins/HugeBag/Resources/midRightBar.tga" },
	  BotRightCorner = { Background = "HabnaPlugins/HugeBag/Resources/botRightCorner.tga" },
	  TopBar = { Background = "HabnaPlugins/HugeBag/Resources/TopBar.tga" },
	  BotBar = { Background = "HabnaPlugins/HugeBag/Resources/BotBar.tga" },
	  TopLeftCorner_over = { Background = "HabnaPlugins/HugeBag/Resources/topLeftCorner_over.tga" },
	  MidLeftBar_over = { Background = "HabnaPlugins/HugeBag/Resources/midLeftBar_over.tga" },
	  BotLeftCorner_over = { Background = "HabnaPlugins/HugeBag/Resources/botLeftCorner_over.tga" },
	  HugeBagName_over = { Image = "HabnaPlugins/HugeBag/Resources/label_over.tga" },
	  TopRightCorner_over = { Background = "HabnaPlugins/HugeBag/Resources/topRightCorner_over.tga" },
	  MidRightBar_over = { Background = "HabnaPlugins/HugeBag/Resources/midRightBar_over.tga" },
	  BotRightCorner_over = { Background = "HabnaPlugins/HugeBag/Resources/botRightCorner_over.tga" }
};

frmMain = class( Turbine.UI.Window );

function frmMain:Constructor()
	Turbine.UI.Window.Constructor( self );

	self.cursor = nil;
	
	--**v Get player backpack v**
	if pcall( Turbine.Gameplay.LocalPlayer.GetInstance ) then
		self.backpack = Turbine.Gameplay.LocalPlayer.GetInstance():GetBackpack();
	else
		self.backpack = Turbine.Gameplay.LocalPlayer():GetBackpack();
	end
	--**^

	backpackSize = self.backpack:GetSize();

	LoadSettings();
	
	--**v Set some settings of HugeBag window v**
	self:SetVisible( true );
	
	WrapperChecker();
	if not HBU then Turbine.PluginManager.LoadPlugin("HugeBag Utilities") end
	if loaded then
		StartVisible = false;
		Turbine.PluginManager.UnloadScriptState( "HugeBagUtilities" );
	end

	if ( StartVisible ) then
		windowOpen = true;
		if WidgetLoc == "right" then
			self:SetLeft( screenWidth - WinWidth );
		elseif WidgetLoc == "left" then
			self:SetLeft( 0 );
		end
	else
		windowOpen = false;
		if WidgetLoc == "right" then
			self:SetLeft( screenWidth - 30 );
		elseif WidgetLoc == "left" then
			self:SetLeft( -420 );
		end
	end
	
	self:SetTop( WinLocY );
	--self:SetOpacity( 1 );
	self:SetAllowDrop( true );
	self:SetWantsKeyEvents( true );
	self:SetZOrder( 1 );
	--**^
	--**v Set Top and size of window v** --
	if backpackSize == 45 then
		self:SetSize( WinWidth, WinHeight - 69 );
	else
		self:SetSize( WinWidth, WinHeight );
	end
	--**^
	--**v Sort control v**
	sortCtr = Turbine.UI.Control();

	sortCtr.Update = function( sender, args )
		local Time = Turbine.Engine.GetGameTime()
		local elapsed = Time - Start
		
		if Slot > backpackSize then
			sortCtr:SetWantsUpdates(false)
			write( "HugeBag: Sorting finished" )
			SortButton:SetVisible( true );
			SearchButton:SetVisible( true );
		else
			--Get the name of the target item
			target = VirtPack[Slot].Name
		
			--Get item in current slot
			item = frmMain.backpack:GetItem(Slot)
			
			--Get the name of the item in the current slot. If empty return that name as "zEmpty"
			if item ~= nil then Name = item:GetName() else Name = "zEmpty" end

			--write("<rgb=#888888>Slot "..Slot.." should contain "..target.." but contains ".. Name)
			
			--If the Name of the current item is equal to the target name, move on.
			if Name == target then Slot = Slot + 1 return end	
			
			if elapsed > 0.2 then
				--Search for the item in the rest of the pack.
				for i = Slot , backpackSize do
					item = frmMain.backpack:GetItem(i)
					if item ~= nil then item = item:GetName() else item = "zEmpty" end
					if item == target then 
						swop( i, Slot)
						Start= Turbine.Engine.GetGameTime()
						return
					end
				end
				write( "Should never get here! Error in Update function." )
				--write("<rgb=#888888>Should never get here! Error in Update function. Did your inventory change during the sort? Restarting!")
				--sortCtr:SetWantsUpdates(false)
				--SortHugeBag()
			end
		end
	end
	--**^
	--**v Slide control v**
	slideCtr = Turbine.UI.Control();

	slideCtr.Update = function( sender, args )
		-- ** Replace if display size has changed
		tscreenWidth, tscreenHeight = Turbine.UI.Display.GetSize();

		if ( tscreenWidth ~= screenWidth ) then
			screenWidth = tscreenWidth;
			if ( windowOpen ) then
				if WidgetLoc == "right" then
					self:SetLeft( screenWidth - WinWidth );
				elseif WidgetLoc == "left" then
					self:SetLeft( 0 );
				end
			else
				if WidgetLoc == "right" then
					self:SetLeft( screenWidth - 30 );
				elseif WidgetLoc == "left" then
					self:SetLeft( -420 );
				end
			end
		end
		-- **

		if WidgetLoc == "right" then
			if ( windowOpen ) then
				if ( self:GetLeft() == ( screenWidth - 30 ) ) then
					slideCtr:SetWantsUpdates( false );
					windowOpen = false;
				else
					self:SetLeft( self:GetLeft() + winRate );
				end
			else
				if ( self:GetLeft() == ( screenWidth - ( winRate * 15 ) ) ) then
					slideCtr:SetWantsUpdates( false );
					windowOpen = true;
				else
					self:SetLeft( self:GetLeft() - winRate );
				end
			end
		elseif WidgetLoc == "left" then
			if ( windowOpen ) then
				if ( self:GetLeft() + self:GetWidth() == 30 ) then
					slideCtr:SetWantsUpdates( false );
					windowOpen = false;
				else
					self:SetLeft( self:GetLeft() - winRate );
				end
			else
				if ( self:GetLeft() + self:GetWidth() == winRate * 15 ) then
					slideCtr:SetWantsUpdates( false );
					windowOpen = true;
				else
					self:SetLeft( self:GetLeft() + winRate );
				end
			end
		end
	end
	--**^
	--**v Top left corner v**
	topLeftCorner = Turbine.UI.Control();
	topLeftCorner:SetParent( self );
	topLeftCorner:SetPosition( 0, 0 );
	topLeftCorner:SetSize( 30, 20 );
	topLeftCorner:SetBackground( resources.TopLeftCorner.Background );

	topLeftCorner.MouseClick = function( sender, args )
		if WidgetLoc == "right" then
			if ( args.Button == Turbine.UI.MouseButton.Right ) then
				menu:ShowMenu()
			end
		end
	end

	topLeftCorner.MouseHover = function( sender, args )
		if WidgetLoc == "right" then
			Loc = "Top";
			MouseHover()
		end
	end

	topLeftCorner.MouseLeave = function ( args )
		if WidgetLoc == "right" then
			MouseLeave()
		end
	end

	topLeftCorner.MouseDown = function( sender, args )
		if WidgetLoc == "right" then
			MouseDown( sender, args )
		end
	end

	topLeftCorner.MouseUp = function( sender, args )
		if WidgetLoc == "right" then
			MouseUp( sender, args )
		end
	end

	topLeftCorner.MouseMove = function( sender, args )
		if WidgetLoc == "right" then
			MouseMove( sender, args )
		end
	end
	--**^
	--**v Middle left bar **
	midLeftBar = Turbine.UI.Control();
	midLeftBar:SetParent( self );
	midLeftBar:SetPosition( 0, 20 );
	midLeftBar:SetSize( 30, self:GetHeight() - 40 );
	midLeftBar:SetBackground( resources.MidLeftBar.Background );

	midLeftBar.MouseEnter = function( sender, args )
		if WidgetLoc == "right" then
			WrapperChecker();
			if not loaded then
				topLeftCorner:SetBackground( resources.TopLeftCorner_over.Background );
				midLeftBar:SetBackground( resources.MidLeftBar_over.Background );
				botLeftCorner:SetBackground( resources.BotLeftCorner_over.Background );
				llabel:SetBackground( resources.HugeBagName_over.Image );
			end
		end
	end
	
	midLeftBar.MouseLeave = function( sender, args )
		if WidgetLoc == "right" then
			WrapperChecker();
			if not loaded then
				topLeftCorner:SetBackground( resources.TopLeftCorner.Background );
				midLeftBar:SetBackground( resources.MidLeftBar.Background );
				botLeftCorner:SetBackground( resources.BotLeftCorner.Background );
				llabel:SetBackground( resources.HugeBagName.Image );
			end
		end
	end

	midLeftBar.MouseClick = function( sender, args )
		if WidgetLoc == "right" then
			WrapperChecker();
			if not loaded then
				if ( args.Button == Turbine.UI.MouseButton.Left ) then
					slideCtr:SetWantsUpdates(true);
				end
			end
		end
	end
	--**^
	--**v Bottom left corner v**
	botLeftCorner = Turbine.UI.Control();
	botLeftCorner:SetParent( self );
	botLeftCorner:SetPosition( 0, self:GetHeight() - 20 );
	botLeftCorner:SetSize( 30, 20 );
	botLeftCorner:SetBackground( resources.BotLeftCorner.Background );

	botLeftCorner.MouseHover = function( sender, args )
		if WidgetLoc == "right" then
			Loc = "Bot";
			MouseHover()
		end
	end

	botLeftCorner.MouseLeave = function ( args )
		if WidgetLoc == "right" then
			MouseLeave()
		end
	end

	botLeftCorner.MouseDown = function( sender, args )
		if WidgetLoc == "right" then
			MouseDown( sender, args )
		end
	end

	botLeftCorner.MouseUp = function( sender, args )
		if WidgetLoc == "right" then
			MouseUp( sender, args )
		end
	end

	botLeftCorner.MouseMove = function( sender, args )
		if WidgetLoc == "right" then
			MouseMove( sender, args )
		end
	end
	--**^
	--**v Top right corner v**
	topRightCorner = Turbine.UI.Control();
	topRightCorner:SetParent( self );
	topRightCorner:SetPosition( self:GetWidth() - 30, 0 );
	topRightCorner:SetSize( 30, 20 );
	topRightCorner:SetBackground( resources.TopRightCorner.Background );

	topRightCorner.MouseClick = function( sender, args )
		if WidgetLoc == "left" then
			if ( args.Button == Turbine.UI.MouseButton.Left ) then
				--slideCtr:SetWantsUpdates(true);
			elseif ( args.Button == Turbine.UI.MouseButton.Right ) then
				menu:ShowMenu()
			end
		end
	end

	topRightCorner.MouseHover = function( sender, args )
		if WidgetLoc == "left" then
			Loc = "Top";
			MouseHover()
		end
	end

	topRightCorner.MouseLeave = function ( args )
		if WidgetLoc == "left" then
			MouseLeave()
		end
	end

	topRightCorner.MouseDown = function( sender, args )
		if WidgetLoc == "left" then
			MouseDown( sender, args )
		end
	end

	topRightCorner.MouseUp = function( sender, args )
		if WidgetLoc == "left" then
			MouseUp( sender, args )
		end
	end

	topRightCorner.MouseMove = function( sender, args )
		if WidgetLoc == "left" then
			MouseMove( sender, args )
		end
	end
	--**^
	--**v Middle right bar v**
	midRightBar = Turbine.UI.Control();
	midRightBar:SetParent( self );
	midRightBar:SetPosition( self:GetWidth() - 30, 20 );
	midRightBar:SetSize( 30, self:GetHeight() - 40 );
	midRightBar:SetBackground( resources.MidRightBar.Background );

	midRightBar.MouseEnter = function( sender, args )
		if WidgetLoc == "left" then
			WrapperChecker();
			if not loaded then
				topRightCorner:SetBackground( resources.TopRightCorner_over.Background );
				midRightBar:SetBackground( resources.MidRightBar_over.Background );
				botRightCorner:SetBackground( resources.BotRightCorner_over.Background );
				rlabel:SetBackground( resources.HugeBagName_over.Image );
			end
		end
	end

	midRightBar.MouseLeave = function( sender, args )
		if WidgetLoc == "left" then
			WrapperChecker();
			if not loaded then
				topRightCorner:SetBackground( resources.TopRightCorner.Background );
				midRightBar:SetBackground( resources.MidRightBar.Background );
				botRightCorner:SetBackground( resources.BotRightCorner.Background );
				rlabel:SetBackground( resources.HugeBagName.Image );
			end
		end
	end

	midRightBar.MouseClick = function( sender, args )
		if WidgetLoc == "left" then
			WrapperChecker();
			if not loaded then
				if ( args.Button == Turbine.UI.MouseButton.Left ) then
					slideCtr:SetWantsUpdates(true);
				end
			end
		end
	end
	--**^
	--**v Bottom right corner v**
	botRightCorner = Turbine.UI.Control();
	botRightCorner:SetParent( self );
	botRightCorner:SetPosition( self:GetWidth() - 30, self:GetHeight() - 20 );
	botRightCorner:SetSize( 30, 20 );
	botRightCorner:SetBackground( resources.BotRightCorner.Background );
	
	botRightCorner.MouseClick = function( sender, args )
		if WidgetLoc == "left" then
			if ( args.Button == Turbine.UI.MouseButton.Left ) then
				--slideCtr:SetWantsUpdates(true);
			end
		end
	end

	botRightCorner.MouseHover = function( sender, args )
		if WidgetLoc == "left" then
			Loc = "Bot";
			MouseHover()
		end
	end

	botRightCorner.MouseLeave = function ( args )
		if WidgetLoc == "left" then
			MouseLeave()
		end
	end

	botRightCorner.MouseDown = function( sender, args )
		if WidgetLoc == "left" then
			MouseDown( sender, args )
		end
	end

	botRightCorner.MouseUp = function( sender, args )
		if WidgetLoc == "left" then
			MouseUp( sender, args )
		end
	end

	botRightCorner.MouseMove = function( sender, args )
		if WidgetLoc == "left" then
			MouseMove( sender, args )
		end
	end
	--**^
	--**v Top bar v**
	TopBar = Turbine.UI.Control();
	TopBar:SetParent( self );
	TopBar:SetPosition( 30, 0 );
	TopBar:SetSize( self:GetWidth() - 60, 30 );
	TopBar:SetBackground( resources.TopBar.Background );
	--**^
	--**v Bottom bar V**
	BotBar = Turbine.UI.Control();
	BotBar:SetParent( self );
	BotBar:SetPosition( 30, self:GetHeight() - 30);
	BotBar:SetSize( self:GetWidth() - 60, 30 );
	BotBar:SetBackground( resources.BotBar.Background );
	-- **^
	--**v Label on left bar v**
	llabel = Turbine.UI.Control();
	llabel:SetParent( midLeftBar );
	llabel:SetZOrder( 10 );
	llabel:SetPosition( 9, ( self:GetHeight() / 2 ) - 62 );
	llabel:SetSize( 14, 88 );
	llabel:SetBackground( resources.HugeBagName.Image );
	llabel:SetBlendMode( Turbine.UI.BlendMode.Overlay );
	llabel:SetMouseVisible( false );
	--**^
	--**v Label on right bar v**
	rlabel = Turbine.UI.Control();
	rlabel:SetParent( midRightBar );
	rlabel:SetZOrder( 10 );
	rlabel:SetPosition( 9, ( self:GetHeight() / 2 ) - 62 );
	rlabel:SetSize( 14, 88 );
	rlabel:SetBackground( resources.HugeBagName.Image );
	rlabel:SetBlendMode( Turbine.UI.BlendMode.Overlay );
	rlabel:SetMouseVisible( false );
	--**^
	--**v Sort button on bottom bar v**
	SortButton = Turbine.UI.Button();
	SortButton:SetParent( BotBar );
	SortButton:SetPosition( 5,  8 );
	SortButton:SetSize( 27, 12 );
	SortButton:SetOutlineColor( black );
	SortButton:SetForeColor( gray );
	SortButton:SetFontStyle( Turbine.UI.FontStyle.Outline );
	SortButton:SetText( "Sort" );

	SortButton.Click = function(sender, args)
		SortButton:SetVisible( false );
		SearchButton:SetVisible( false );
		SortHugeBag()
	end

	SortButton.MouseHover = function(sender, args)
		SortButton:SetForeColor( green );
	end

	SortButton.MouseLeave = function(sender, args)
		SortButton:SetForeColor( gray );
	end
	--**^
	--**v Search button on bottom bar v**
	SearchButton = Turbine.UI.Button();
	SearchButton:SetParent( BotBar );
	SearchButton:SetPosition( 40, 8 );
	SearchButton:SetSize( 43, 12 );
	SearchButton:SetOutlineColor( black );
	SearchButton:SetForeColor( gray );
	SearchButton:SetFontStyle( Turbine.UI.FontStyle.Outline );
	SearchButton:SetText( "Search" );

	SearchButton.Click = function(sender, args)
		SortButton:SetVisible( false );
		SearchButton:SetVisible( false );
		SearchInput:SetVisible( not SearchInput:IsVisible() );
		--SearchInput:SetFocus( true );
	end

	SearchButton.MouseHover = function(sender, args)
		SearchButton:SetForeColor( green );
	end

	SearchButton.MouseLeave = function(sender, args)
		SearchButton:SetForeColor( gray );
	end
	--**^
	--**v Text box on bottom bar v**
	SearchInput = Turbine.UI.Lotro.TextBox();
	SearchInput:SetParent( BotBar );
	SearchInput:SetPosition( 5, 4 );
	SearchInput:SetSize( 80, 20 );
	SearchInput:SetMultiline( false );
	SearchInput:SetFont( Turbine.UI.Lotro.Font.TrajanPro14 );
	SearchInput.Text = "";
	SearchInput:SetText( SearchInput.Text );
	SearchInput:SetWantsKeyEvents( true );
	SearchInput:SetWantsUpdates( false );
	SearchInput:SetVisible( false );

	SearchInput.KeyDown = function( sender, args )
		if ( args.Action == 19 ) then -- right mouse button click
			SearchInput:SetWantsUpdates( false );
			SearchInput:SetText( "" );
			SearchInput:SetVisible( false );
			SortButton:SetVisible( true );
			SearchButton:SetVisible( true );
		end
		Search();
	end

	SearchInput.FocusGained = function()
		SearchInput:SetWantsUpdates( true );
	end
	
	SearchInput.FocusLost = function()
		SearchInput:SetWantsUpdates( false );
		SearchInput:SetText( "" );
		SearchInput:SetVisible( false );
		SortButton:SetVisible( true );
		SearchButton:SetVisible( true );
		Search();
	end
	
	SearchInput.Update = function()
		if SearchInput.Text ~= SearchInput:GetText() then
			SearchInput.Text = SearchInput:GetText()
			Search();
		end
	end
	--**^
	--**v Slots informations on top bar v**
	lblTitle = Turbine.UI.Label();
	lblTitle:SetParent( self );
	lblTitle:SetSize( self:GetWidth() - 60, 30 );
	lblTitle:SetPosition( 30, 2 );
	lblTitle:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
	lblTitle:SetFont( Turbine.UI.Lotro.Font.TrajanPro19 );
	lblTitle:SetForeColor( rustedgold );
	lblTitle:SetOutlineColor( black );
	lblTitle:SetFontStyle( Turbine.UI.FontStyle.Outline );
	lblTitle:SetBlendMode( Turbine.UI.BlendMode.Overlay );
	--**^
	--**v Set some settings of the itemlistbox v**
	itemListBox = Turbine.UI.ListBox();
	itemListBox:SetParent( self );
	itemListBox:SetOrientation( Turbine.UI.Orientation.Horizontal );
	itemListBox:SetPosition( InitXPos, InitYPos );
	itemListBox:SetBackColor( Turbine.UI.Color( bcAlpha, bcRed, bcGreen, bcBlue ) );
	itemListBox:SetMaxItemsPerLine( 11 );
	--itemListBox:SetBlendMode( Turbine.UI.BlendMode.Overlay );
	
	if backpackSize == 45 then
		itemListBox:SetSize( 389, 180 ); -- to show 45 slots
	else
		itemListBox:SetSize( 389, 249 ); -- to show 75 slots
	end
	--**^
	--**v Backpack v**
	self:SetBackPack(); --> ** Set the backpack **

	self.backpack.ItemAdded = function( sender, args )
		items[args.Index]:SetItem( self.backpack:GetItem( args.Index ) );
		freeslots = freeslots -1;
		self:PrintTitle()
	end
	
	self.backpack.ItemRemoved = function( sender, args )
		items[args.Index]:SetItem( self.backpack:GetItem( args.Index ) );
		freeslots = freeslots +1;
		self:PrintTitle()
	end
	
	self.backpack.ItemMoved = function( sender, args )
		items[args.OldIndex]:SetItem( self.backpack:GetItem( args.OldIndex ) );
		items[args.NewIndex]:SetItem( self.backpack:GetItem( args.NewIndex ) );
	end
	--**^

	self.KeyDown = function( sender, args )
		if ( args.Action == Turbine.UI.Lotro.Action.Escape ) then
			if ( HideWEsc ) then
				if ( windowOpen ) then
					slideCtr:SetWantsUpdates(true);
				end
			else
				self:SetVisible( true ); -- **v DO NOT Hide HugeBag if ESC key is press v**
			end
		end
		
		if ( args.Action == Turbine.UI.Lotro.Action.ToggleBags or
		     args.Action == Turbine.UI.Lotro.Action.ToggleBag1 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag2 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag3 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag4 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag5 )
		then
			slideCtr:SetWantsUpdates(true);
		end
	end

	self.DragDrop = function( sender, args )
		local shortcut = args.DragDropInfo:GetShortcut();
		local destinationItemControl = itemListBox:GetItemAt( args.X - InitXPos, args.Y - InitYPos );
		
		if ( destinationItemControl ~= nil ) then -- ** if nil, then it's out of range, so don't do anything **
			local destinationIndex = itemListBox:IndexOfItem( destinationItemControl );
			self.backpack:PerformShortcutDrop(shortcut, destinationIndex, Turbine.UI.Control.IsShiftKeyDown() );
		end
	end
end

--**v Set the backpack v**
function frmMain:SetBackPack()
	items = nil;
	itemListBox:ClearItems();
	freeslots = 0;
	
	items = {};
	
	for i = 1, backpackSize do
		itemsCtl = Turbine.UI.Control();
		itemsCtl.itemIndex = i;
		itemsCtl:SetSize( 35, 35 );
		itemsCtl:SetBackground ( resources.Item.Background );
		itemsCtl:SetBlendMode( Turbine.UI.BlendMode.Overlay );

		items[i] = Turbine.UI.Lotro.ItemControl( self.backpack:GetItem( i ) );
		items[i]:SetParent( itemsCtl );

		itemListBox:AddItem( itemsCtl );

		if ( self.backpack:GetItem( i ) == nil ) then
			freeslots = freeslots + 1;
		end
	end
	
	frmMain:PrintTitle()
end
--**^
--**v Print the title with the free slots and the max v**
function frmMain:PrintTitle()
	local usedslots = backpackSize - freeslots;
	lblTitle:SetText( "Free slots: " .. freeslots .. "  Used slots: " .. usedslots .. "  Max slots: " .. backpackSize );
end
--**^