-- frmMain.lua
-- Originally TheOneBagWindow.lua written by Rod, extended by Habna


import "HabnaPlugins.HugeBag.settings"; -- LUA settings file
import "HabnaPlugins.HugeBag.functions"; -- LUA functions file
import "HabnaPlugins.HugeBag.color"; -- LUA color file
import "HabnaPlugins.HugeBag.GlobalVariable"; -- LUA global variable file
import "HabnaPlugins.HugeBag.sortingList"; -- LUA sorting list file
import "HabnaPlugins.HugeBag.options"; -- LUA options file

local resources = {
	  Item = { Background = "HabnaPlugins/HugeBag/Resources/slotBackground.tga" },
	  Resize = { Background = "HabnaPlugins/HugeBag/Resources/resize.tga" }
};

frmMain = class( Turbine.UI.Lotro.Window );
--frmMain = class( Turbine.UI.Lotro.GoldWindow ); --Gold Window

function frmMain:Constructor()
	--v SETTINGS SECTION v--
	Turbine.UI.Lotro.Window.Constructor( self );
	--Turbine.UI.Lotro.GoldWindow.Constructor( self ); --Gold Window
	
	self.cursor = nil;

	LoadSettings();
	
	sortCtr = Turbine.UI.Control();
	
	if not HBU then Turbine.PluginManager.LoadPlugin("HugeBag Utilities") end
	if loaded then
		self:SetVisible( false );
		Turbine.PluginManager.UnloadScriptState( "HugeBagUtilities" );
	else
		self:SetVisible( StartVisible );
	end

	--**v Set HugeBag window v**
	self:SetSize( WinWidth, WinHeight );
	self:SetPosition( WinLocX, WinLocY );
	self:SetAllowDrop( true );
	self:SetWantsKeyEvents( true );
	self:SetZOrder( 1 );
	--**^
	--**v Lock Label v**
	LockLabel = Turbine.UI.Control();
	LockLabel:SetParent( self );
	LockLabel:SetPosition( 22, 0 );
	LockLabel:SetSize( WinWidth - 50, 33 );
	if ( LockPosition ) then
		LockLabel:SetVisible( true );
	else
		LockLabel:SetVisible( false );
	end
	--**^
	-- **v Sort button v**
	SortButton = Turbine.UI.Button();
	SortButton:SetParent( self );
	SortButton:SetText( "Sort" );
	SortButton:SetSize( 27, 12 );
	SortButton:SetForeColor( gray );
	SortButton:SetOutlineColor( black );
	SortButton:SetFontStyle( Turbine.UI.FontStyle.Outline );

	SortButton.Click = function(sender, args)
		SortButton:SetVisible( false );
		SearchButton:SetVisible( false );
		SortHugeBag()
	end

	SortButton.MouseHover = function(sender, args)
		SortButton:SetForeColor( green );
	end

	SortButton.MouseLeave = function(sender, args)
		SortButton:SetForeColor( gray );
	end
	-- **^
	-- **v Search button v**
	SearchButton = Turbine.UI.Button();
	SearchButton:SetParent( self );
	SearchButton:SetText( "Search" );
	SearchButton:SetSize( 43, 12 );
	SearchButton:SetForeColor( gray );
	SearchButton:SetOutlineColor( black );
	SearchButton:SetFontStyle( Turbine.UI.FontStyle.Outline );

	SearchButton.Click = function(sender, args)
		SearchInput:SetVisible( not SearchInput:IsVisible() );
		SortButton:SetVisible( false );
		SearchButton:SetVisible( false );
		--SearchInput:SetFocus( true ); --Try to find a way to put focus in the box
	end

	SearchButton.MouseHover = function(sender, args)
		SearchButton:SetForeColor( green );
	end

	SearchButton.MouseLeave = function(sender, args)
		SearchButton:SetForeColor( gray );
	end
	--**^
	-- **v Set the scrollbar v**
	self.itemListBoxScrollBar = Turbine.UI.Lotro.ScrollBar();
	self.itemListBoxScrollBar:SetOrientation( Turbine.UI.Orientation.Vertical );
	self.itemListBoxScrollBar:SetParent( self );
	-- **^
	-- **v Set the itemlistbox v**
	self.itemListBox = Turbine.UI.ListBox();
	self.itemListBox:SetParent( self );
	self.itemListBox:SetOrientation( Turbine.UI.Orientation.Horizontal );
	self.itemListBox:SetVerticalScrollBar( self.itemListBoxScrollBar );
	self.itemListBox:SetBackColor( Turbine.UI.Color( bcAlpha, bcRed, bcGreen, bcBlue ) );
	-- **^
	-- **v Search box v**
	SearchInput = Turbine.UI.Lotro.TextBox();
	SearchInput:SetParent( self );
	SearchInput.Text = "";
	SearchInput:SetSize( 80, 20 );
	SearchInput:SetMultiline( false );
	SearchInput:SetFont( Turbine.UI.Lotro.Font.TrajanPro14 );
	SearchInput:SetText( SearchInput.Text );
	SearchInput:SetWantsKeyEvents( true );
	SearchInput:SetWantsUpdates( false );
	SearchInput:SetVisible( false );

	SearchInput.KeyDown = function( sender, args )
		if ( args.Action == 19 ) then -- right mouse button click
			SearchInput:SetWantsUpdates( false );
			SearchInput:SetText( "" );
			SearchInput:SetVisible( false );
			SortButton:SetVisible( true );
			SearchButton:SetVisible( true );
		end
		Search();
	end
	
	SearchInput.FocusGained = function()
		SearchInput:SetWantsUpdates( true );
	end
	
	SearchInput.FocusLost = function()
		SearchInput:SetWantsUpdates( false );
		SearchInput:SetText( "" );
		SearchInput:SetVisible( false );
		SortButton:SetVisible( true );
		SearchButton:SetVisible( true );
		Search();
	end
	
	SearchInput.Update = function()
		if SearchInput.Text ~= SearchInput:GetText() then
			SearchInput.Text = SearchInput:GetText()
			Search();
		end
	end
	-- **^
	-- **v Get player gameplay v**
	if pcall( Turbine.Gameplay.LocalPlayer.GetInstance ) then
		self.backpack = Turbine.Gameplay.LocalPlayer.GetInstance():GetBackpack();
	else
		self.backpack = Turbine.Gameplay.LocalPlayer():GetBackpack();
	end
	-- **^

	backpackSize = self.backpack:GetSize();
	self:SetBackPack(); --> ** Set the backpack **
	
	self.MouseDown = function( sender, args )
		if ( args.Button == Turbine.UI.MouseButton.Left ) then
			if ( args.X < 30 and args.Y > 15 and args.Y < 50 )then
				CornerClick = "TopLeft";
				--frmSettings()
			elseif ( ( args.X < 30 ) and ( args.Y > ( self:GetHeight() - 30 ) ) ) then
				CornerClick = "BottomLeft";
				dragStartX = args.X;
				dragStartY = args.Y;
				dragging = true;
			elseif ( args.X > ( self:GetWidth() -30 ) ) and ( args.Y > ( self:GetHeight() - 30 ) ) then
				CornerClick = "BottomRight";
				dragStartX = args.X;
				dragStartY = args.Y;
				dragging = true;
			end
		elseif ( args.Button == Turbine.UI.MouseButton.Right ) then
			if ( args.X < 30 and args.Y > 15 and args.Y < 50 )then
				frmOptions();
			end
		end
		-- ** To prevent from lock the context menu, le lock must nullify the dragging
		if ( LockSize ) then
			dragging = false;
		end
	end
	
	self.MouseUp = function( sender, args )
		dragging = false;
		OutOfBound = false;
		CornerClick = "";
		SaveSettings()
	end
	
	self.MouseMove = function( sender, args )
		if ( dragging ) then
			if self.cursor ~= nil then
				self.cursor:SetVisible( false );
				self.cursor = nil;
			end

			WinWidth, WinHeight = self:GetSize();
			WinLocX, WinLocY = self:GetPosition();
			local screenWidth = Turbine.UI.Display:GetWidth();
			local screenHeight = Turbine.UI.Display:GetHeight();
			local tempY = WinHeight + ( args.Y - dragStartY );

			if CornerClick == "BottomRight" then
				-- **v Check horizontal v**
				local tempX = WinWidth + ( args.X - dragStartX );
				if ( WinLocX + tempX ) > screenWidth then
					self:SetSize( ( screenWidth - WinLocX ), WinHeight );
				else
					self:SetSize( tempX, tempY );
					dragStartX = dragStartX + (args.X - dragStartX);
					dragStartY = dragStartY + (args.Y - dragStartY);
				end
				-- **^
			elseif CornerClick == "BottomLeft" then
				-- **v Check horizontal v**
				local tempX = WinWidth - ( args.X - dragStartX );
				if WinLocX < 0 then
					dragging = false;
					self:SetPosition( 0, WinLocY );
					--self:SetSize( WinWidth , WinHeight );
				else
					self:SetPosition( WinLocX + ( args.X - dragStartX ), WinLocY );
					self:SetSize( tempX, tempY );
					dragStartY = dragStartY + ( args.Y - dragStartY );
				end
				-- **^
			end
			-- **v Check vertical v**
			if ( WinLocY + tempY ) > screenHeight then
				self:SetSize( WinWidth, ( screenHeight - WinLocY ) );
			end
			-- **^
			self:PerformLayout()
		else
			if ( args.X < 30 and args.Y > 15 and args.Y < 50 )then
				-- "TopLeft"
				self.cursor = rightclicCursor;
				local mouseX, mouseY = Turbine.UI.Display.GetMousePosition();
				self.cursor:SetPosition( mouseX - self.cursor.xOffset, mouseY - self.cursor.yOffset);
				self.cursor:SetVisible( true );
			elseif ( ( args.X < 30 ) and ( args.Y > ( self:GetHeight() - 30 ) ) ) then
				-- "BottomLeft"
				ShowResizeCursor();
			elseif ( args.X > ( self:GetWidth() -30 ) ) and ( args.Y > ( self:GetHeight() - 30 ) ) then
				-- "BottomRight"
				ShowResizeCursor();
			else
				if self.cursor ~= nil then
					self.cursor:SetVisible( false );
					self.cursor = nil;
				end
			end
		end
		self:PrintTitle();
	end

	self.MouseLeave = function( sender, args )
		if self.cursor ~= nil then
			self.cursor:SetVisible( false );
			self.cursor = nil;
		end
	end
	
	self.Closing = function( sender, args )
		
	end
	
	self.PositionChanged = function(args)
		WinLocX, WinLocY = self:GetPosition()
		SaveSettings()
	end
	
	self.KeyDown = function( sender, args )
		if ( args.Action == Turbine.UI.Lotro.Action.Escape ) then
			if ( HideWEsc ) then
				self:SetVisible( false ); -- **v Hide HugeBag if ESC key is press v**
			end
		end
		
		if ( args.Action == Turbine.UI.Lotro.Action.ToggleBags or
		     args.Action == Turbine.UI.Lotro.Action.ToggleBag1 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag2 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag3 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag4 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag5 )
		then
			sender:SetVisible( not sender:IsVisible() ) -- **v Show/Hide HugeBag if any bag was toggle v**
		end
	end
	
	
	sortCtr.Update = function( sender, args )
		local Time = Turbine.Engine.GetGameTime()
		local elapsed = Time - Start
		
		if Slot > backpackSize then
			sortCtr:SetWantsUpdates(false)
			write( "HugeBag: Sorting finished" )
			SortButton:SetVisible( true );
			SearchButton:SetVisible( true );
		else
			--Get the name of the target item
			target = VirtPack[Slot].Name
		
			--Get item in current slot
			item = frmMain.backpack:GetItem(Slot)
			
			--Get the name of the item in the current slot. If empty return that name as "zEmpty"
			if item ~= nil then Name = item:GetName() else Name = "zEmpty" end

			--write("<rgb=#888888>Slot "..Slot.." should contain "..target.." but contains ".. Name)
			
			--If the Name of the current item is equal to the target name, move on.
			if Name == target then Slot = Slot + 1 return end	
			
			if elapsed > 0.2 then
				--Search for the item in the rest of the pack.
				for i = Slot , backpackSize do
					item = frmMain.backpack:GetItem(i)
					if item ~= nil then item = item:GetName() else item = "zEmpty" end
					if item == target then 
						swop( i, Slot)
						Start= Turbine.Engine.GetGameTime()
						return
					end
				end
				write( "Should never get here! Error in Update function." )
				--write("<rgb=#888888>Should never get here! Error in Update function. Did your inventory change during the sort? Restarting!")
				--sortCtr:SetWantsUpdates(false)
				--SortHugeBag()
				SortButton:SetVisible( true );
			end
		end
	end
	
	self.backpack.SizeChanged = function( sender, args )
		self:SetBackPack();
	end
	
	self.backpack.ItemAdded = function( sender, args )
		self.items[args.Index]:SetItem( self.backpack:GetItem( args.Index ) );
		freeslots = freeslots -1;
		self:PrintTitle()
	end
	
	self.backpack.ItemRemoved = function( sender, args )
		self.items[args.Index]:SetItem( self.backpack:GetItem( args.Index ) );
		freeslots = freeslots +1;
		self:PrintTitle()
	end
	
	self.backpack.ItemMoved = function( sender, args )
		self.items[args.OldIndex]:SetItem( self.backpack:GetItem( args.OldIndex ) );
		self.items[args.NewIndex]:SetItem( self.backpack:GetItem( args.NewIndex ) );
	end

	self.DragDrop = function( sender, args )
		local shortcut = args.DragDropInfo:GetShortcut();
		local destinationItemControl = self.itemListBox:GetItemAt( args.X - InitXPos, args.Y - InitYPos );
		
		if ( destinationItemControl ~= nil ) then -- ** if nil, then it's out of range, so don't do anything **
			local destinationIndex = self.itemListBox:IndexOfItem( destinationItemControl );
			self.backpack:PerformShortcutDrop(shortcut, destinationIndex, Turbine.UI.Control.IsShiftKeyDown() );
		end
		
	end
end

-- **v Refresh the backpack list v**
function frmMain:SetBackPack()
	self.items = nil;
	self.itemListBox:ClearItems();
	freeslots = 0;

	self.items = {};

	for i = 1, backpackSize, 1 do
		itemsCtl = Turbine.UI.Control();
		itemsCtl.itemIndex = i;
		itemsCtl:SetSize( 35, 35 );
		itemsCtl:SetBackground( resources.Item.Background );
		itemsCtl:SetBlendMode( Turbine.UI.BlendMode.Overlay );

		self.items[i] = Turbine.UI.Lotro.ItemControl( self.backpack:GetItem( i ) );
		self.items[i]:SetParent( itemsCtl );

		self.itemListBox:AddItem( itemsCtl );

		if ( self.backpack:GetItem( i ) == nil ) then
			freeslots = freeslots + 1;
		end
	end
	
	self:PerformLayout();
	self:PrintTitle()
end
-- **^
-- **v Layout of the items in the listbox v**
function frmMain:PerformLayout()
	WinWidth, WinHeight = self:GetSize();
	WinLocX, WinLocY = self:GetPosition()
	local itemWidth = 40;

	if ( self.items[1] ~= nil ) then
		itemWidth = self.items[1]:GetWidth()
	end

	local listWidth = WinWidth - 37;
	local listHeight = WinHeight - 52;
	local itemsPerRow = listWidth / itemWidth;
	
	self.itemListBox:SetPosition( InitXPos, InitYPos );
	self.itemListBox:SetSize( listWidth, listHeight );
	self.itemListBox:SetMaxItemsPerLine( itemsPerRow );
	
	self.itemListBoxScrollBar:SetPosition( WinWidth - 20, 40 );
	self.itemListBoxScrollBar:SetSize( 10, listHeight - 15 );
	
	LockLabel:SetSize(WinWidth - 50, 33 ); --> ** Resize width the LockLabel with HugeBag width **
	SortButton:SetPosition( 35,  self:GetHeight() - 13 ); --> ** Replace 'Sort Bag' button with HugeBag height **
	SearchButton:SetPosition( 70,  self:GetHeight() - 13 ); --> ** Replace 'Sort Bag' button with HugeBag height **
	SearchInput:SetPosition( 35,  self:GetHeight() - 20 ); --> ** Replace 'Search Box' with HugeBag height **
end
-- **^
-- **v Print the title with the free slots and the max v**
function frmMain:PrintTitle()
	if ( ShowSlotsInfo ) then
		if ( ToggleUsed ) then 
			local usedslots = backpackSize - freeslots;
			if WinWidth < 225 then
				self:SetText( usedslots .. " / " .. backpackSize );
			elseif WinWidth < 325 then
				self:SetText( "used slots: " .. usedslots .. " / " .. backpackSize );
			else
				self:SetText( "HugeBag - used slots: " .. usedslots .. " / " .. backpackSize );
			end
		else
			if WinWidth < 225 then
				self:SetText( freeslots .. " / " .. backpackSize );
			elseif WinWidth < 325 then
				self:SetText( "free slots: " .. freeslots .. " / " .. backpackSize );
			else
				self:SetText( "HugeBag - free slots: " .. freeslots .. " / " .. backpackSize );
			end
		end
	else
		if WinWidth < 165 then
			self:SetText( "HB" );
		else
			self:SetText( "HugeBag" );
		end
	end
end
-- **^